<?php

# for debug
# ini_set('display_errors', 1);
# error_reporting(-1);
# ---
class SimpleCounter extends Plugin {

    private $datafile_name = "data.txt";
    private $datafile_path_rel;
    private $datafile_path_abs;
	
    public function __construct() {
        parent::__construct();
        date_default_timezone_set('Europe/Berlin'); // default
        $this->datafile_path_rel = basename(dirname(__FILE__)).'/'.$this->datafile_name;
        $this->datafile_path_abs = PLUGIN_DIR_REL.$this->datafile_path_rel;
		
		//create file if not exists
		if (!file_exists($this->datafile_path_abs)){			
			file_put_contents ($this->datafile_path_abs,"");
		}
    }

    // ----------------------------------------------------------------------
    // getContent()
    // ----------------------------------------------------------------------
	function getContent($value) { 	
		
		$start_time = $this->msec();
		$current_ip = $this->getIP(); 
		$hostname = gethostbyaddr($current_ip);
		
		$sdebug = "";
		$lang = "de";
		if (isset($_SESSION["cmslanguage"])){
				if (substr($_SESSION["cmslanguage"],0,2) != "de"){
					$lang = "en";
				}
		}				
		
		$sStatistics = "statistics";
		$sOnline = "online";
		$sToday = "today";
		$sWeek = "week";
		$sMonth = "month";
		$sYear = "year";
		$sTotal = "total";
		$max = "max.";
		$isBot = false;
		
		if ($lang =="de")
		{		
			$sStatistics = "Statistik";
			$sOnline = "Online";
			$sToday = "Heute";
			$sWeek = "Woche";
			$sMonth = "Monat";
			$sYear = "Jahr";
			$sTotal = "Gesamt";
			$max = "Max.";							
		}
		
        // dauer ip-sperre in sek.
        $ip_lock_sec = (int) $this->settings->get("ip_lock_sec");
        
        // user so lange (sek.) als online anführen nach letzen request
        $ip_user_online_sec = (int) $this->settings->get("ip_user_online_sec");

        // flag: wird der aktuelle client ignoriert oder gezählt
        $count_act_client = true;
        $current_time = time(); // $_SERVER['REQUEST_TIME'];
        
        
        // ip als md5() ablegen (wegen ip-logging-datenschutz)
        #$current_ip = $current_ip; //md5($current_ip);
        
        // bots ignorieren
        $current_browser = (isset($_SERVER['HTTP_USER_AGENT'])) ? addslashes(trim($_SERVER['HTTP_USER_AGENT'])) : "";
		$botlist = array ( "bot","spider", "crawl", "search" ); //"compute-1.amazonaws.com", "yahoo", "kyivstar.net", "sovam.net.ua"
		
        foreach($botlist as $bot){
			if(stripos($current_browser,$bot)!==false){
				$count_act_client = false;	// Is a bot
				$isBot = true;
				break;
			}
		}
						 	
		if ($isBot == false){
			$botlist_byhostname = array ( "amazonaws.com", "kyivstar.net", "sovam.net.ua" );
			foreach($botlist_byhostname as $bot){
				if(stripos($hostname,$bot)!==false){
					$count_act_client = false;	// Is a bot
					$isBot = true;
					break;
				}
			}
		}
		
        // wenn cookie vorhanden, ignorieren
        if (isset($_COOKIE['SimpleCounter'])) {
            $count_act_client = false;
        }
		else{
			// cookie updaten/setzen
			setcookie('SimpleCounter', $current_ip, time() + $ip_lock_sec, '/');
		}
		
        // file öffnen
        $fh = null;
        $fh = fopen($this->datafile_path_abs, 'r+');

        if (!is_resource($fh)) {
            return $this->buildErrorMsg('Konnte die Datei "'.$this->datafile_path_rel.'" nicht &ouml;ffnen!');
        }

        // sperren
        if (!flock($fh, LOCK_EX)) {
        	return $this->buildErrorMsg('Sperren der Datei "'.$this->datafile_path_rel.'" fehlgeschlagen!');
        }

        $size = null;
        $size = filesize($this->datafile_path_abs);
        
        if ($size === null OR $size === false) {
            return $this->buildErrorMsg('Ungültiger Zugriff beim ermitteln der Dateigröße von "'.$this->datafile_path_rel.'"!');
        
        } elseif ($size < 5) {
            // file leer (ein leeres serialisiertes array ist (string) 6 )
            // array mit defaults erstellen (gezählt wird weiter unten)
            $arr_counter_data = array(
                    "day"   => date("z"),
                    "week"  => date("W"),
                    "month" => date("n"),
                    "year"  => date("Y"),
                    "dayHits"    => 1,
                    "weekHits"   => 1, 
                    "monthHits"  => 1,
                    "yearHits"   => 1,
                    "allHits"    => 1,
                    "maxDayHits" => 1,
                    "maxDayHits_timestamp" => $current_time,
                    "arrClientIDs" => array($current_ip => $current_time)
                );

        } else {
            // inhalt einlesen
            // normale verarbeitung
            $file_data = fread($fh, $size);
            // array aus file erstellen
            $arr_counter_data = array();
            $arr_counter_data = unserialize($file_data);
            if ($arr_counter_data === false) {
            	return $this->buildErrorMsg('Inhalt von "'.$this->datafile_path_rel.'" ist nicht interpretierbar!');
            }

        }
		
        if ($count_act_client){
			// hat diese client-id schon einen eintrag in den letzten expire sec gehabt, dann ignonieren
			if ( isset($arr_counter_data['arrClientIDs_FirstStamp'][$current_ip]) && $current_time - $ip_lock_sec < $arr_counter_data['arrClientIDs_FirstStamp'][$current_ip] ) {							
				$count_act_client = false;
			}
        }
        // frist geht immer vom letzten request weg. (wäre Falsch wenn die IP-Sperre 84600 (1Tag) wäre - > erster besuch 8:00 Uhr ( 1x gezählt), nächster //20:00, weiter 8:00, 20:00 usw. keine weitere Zählungen mehr, deswegen wurde  zusätzlich arrClientIDs_FirstStamp eingefügt
		$arr_counter_data['arrClientIDs'][$current_ip] = $current_time;       
							
        // counter hochzählen
        if ($count_act_client) {
			// Zeit des ersten Besuchs da in der arrClientIDs wird die Zeit immer wieder aktualisiert
			$arr_counter_data['arrClientIDs_FirstStamp'][$current_ip] = $current_time; 	
            // tag
            if ($arr_counter_data['day'] == date("z")) {
            	$arr_counter_data['dayHits']++;
            } else {
            	$arr_counter_data['day'] = date("z");
                $arr_counter_data['dayHits'] = 1;
            }
        
            // woche
            if ($arr_counter_data['week'] == date("W")) {
            	$arr_counter_data['weekHits']++;
            } else {
                $arr_counter_data['week'] = date("W");
            	$arr_counter_data['weekHits'] = 1;
            }
        
            // monat
            if ($arr_counter_data['month'] == date("n")) {
            	$arr_counter_data['monthHits']++;
            } else {	
            	$arr_counter_data['month'] = date("n");
            	$arr_counter_data['monthHits'] = 1;
            }
        
            // jahr
            if ($arr_counter_data['year'] == date("Y")) {
            	$arr_counter_data['yearHits']++;
            } else {    	
        		$arr_counter_data['year'] = date("Y");
        		$arr_counter_data['yearHits'] = 1;
        	}
        
            // alle hits
            $arr_counter_data['allHits']++;
        
            // neuer max-rekord?
            if ($arr_counter_data['dayHits'] > $arr_counter_data['maxDayHits']) {
                $arr_counter_data['maxDayHits'] = $arr_counter_data['dayHits'];
                $arr_counter_data['maxDayHits_timestamp'] = $current_time;
            }
        }
        else{
			// Tag aktualisieren, falls nicht stimmt, sonst nach Mitternacht werden weiter Daten vom Vortag gezeigt
			if ($arr_counter_data['day'] != date("z")){
				$arr_counter_data['day'] = date("z");	
				$arr_counter_data['dayHits'] = 0;
				
				// check week, month and year only when day changes, is much performant
				if ($arr_counter_data['week'] != date("W")) {
					$arr_counter_data['week'] = date("W");
					$arr_counter_data['weekHits'] = 0;
				}
				
				if ($arr_counter_data['month'] != date("n")) {           
					$arr_counter_data['month'] = date("n");
					$arr_counter_data['monthHits'] = 0;
				}
				
				if ($arr_counter_data['year'] != date("Y")) {            	
					$arr_counter_data['year'] = date("Y");
					$arr_counter_data['yearHits'] = 0;
				}		
			}
						
		}
        // arrClientIDs aufräumen und aktuelle anzahl online user bestimmen
        $users_online = 0;
        
        foreach ($arr_counter_data['arrClientIDs'] as $id => $timestamp) {
            
            // $ip_user_online_sec abgelaufen
            if ($timestamp + $ip_user_online_sec < $current_time) {
                // $ip_lock_sec abgelaufen? dann user aus sperrliste löschen
                if ($timestamp + $ip_lock_sec < $current_time) {
                    unset($arr_counter_data['arrClientIDs'][$id]);
                }
            } else {
                // $ip_user_online_sec nicht abgelaufen, anzahl erhöhen
                $users_online++;
            }
        }

        # DEBUG
        # echo "<hr><pre>".print_r($arr_counter_data, true)."</pre><hr>";
        
        // Daten speichern
        rewind($fh);                               // zeiger zu beginn
        ftruncate($fh, 0);                         // inhalt löschen
        fwrite($fh, serialize($arr_counter_data)); // aktuelle daten schreiben
        fflush($fh);                               // buffer forcen
        flock($fh, LOCK_UN);                       // lock aufheben
        fclose($fh);                               // handle schliessen
        
        // im "hidden-mode" nur leerstring zurückgeben
        if ($value == "hidden") return "";
        
        //---------------------------------------
        // Ausgabe
        //---------------------------------------
        
        // aufzählungszeichen auslesen und leerzeichen dran hängen
        $list_prefix = $this->settings->get("list_prefix") . " ";

        // textfeld userdefined?
        $html = $this->settings->get("counter_userdefined");
        
        // default darstellung wenn nix angegeben
        if (empty($html)) {
            $html = "{IconUndTitel|".$sStatistics."}
                     {UserOnline}
                     {Heute}
                     {Woche}
                     {Monat}
                     {Jahr}
                     {Gesamt}
                     {Rekord}";
        }
        
        // icon html, ohne <li>
        $icon_html = '<img class="sc_icon" src="'.URL_BASE.PLUGIN_DIR_NAME.'/SimpleCounter/icon.png" alt="Statistik" title="Statistik" />';

        // zuerst die Platzhalter mit möglichem benutzerdef. Text

        // {IconUndTitel|Benutzerdef. Titel}
        $html = $this->replacePlaceholder($html, "IconUndTitel", $icon_html, "sc_title");
        
        // {Titel} oder {Titel|Benutzerdef. Titel}
        $html = $this->replacePlaceholder($html, "Titel", "", "sc_title");
       
        // und dann noch die normalen platzhalter ohne optionalen values
        $arr_user_vars = array(
            "{UserOnline}",
            "{Heute}",
            "{Woche}",
            "{Monat}",
            "{Jahr}",
            "{Gesamt}",
            "{Rekord}",
        );
        $arr_user_repl = array(
            '<li class="sc_online">'.$list_prefix.$users_online . ' '.$sOnline.'</li>',
            '<li class="sc_heute">' .$list_prefix.$this->formatNumber($arr_counter_data['dayHits'])  .' '.$sToday.'</li>',
            '<li class="sc_woche">' .$list_prefix.$this->formatNumber($arr_counter_data['weekHits']) .' '.$sWeek.'</li>',
            '<li class="sc_monat">' .$list_prefix.$this->formatNumber($arr_counter_data['monthHits']).' '.$sMonth.'</li>',
            '<li class="sc_jahr">'  .$list_prefix.$this->formatNumber($arr_counter_data['yearHits']) .' '.$sYear.'</li>',
            '<li class="sc_gesamt">'.$list_prefix.$this->formatNumber($arr_counter_data['allHits'])  .' '.$sTotal.'</li>',
            '<li class="sc_rekord">' .$max . ': ' .$this->formatNumber($arr_counter_data['maxDayHits']). " (" . date("d.m.Y", $arr_counter_data['maxDayHits_timestamp']). ')</li>'
        );
        
        // Platzhalter ersetzen
        $html = '<ul class="sc_counter">'.str_replace($arr_user_vars, $arr_user_repl, $html).'</ul>' ;

        // etwaige umbrüche entfernen, da diese als <br /> ausgegeben werden
        $html = str_replace("<br />", "", $html);

        // -------------------
		
		$stop_time = $this->msec();
		$duration =  $stop_time - $start_time;
		$host = "hostname: " . $hostname . "<br>";
		//$html .= "<div class=\"sysinfoinner\">" . $host . "exec. time: " . $duration . " ms.</div>";
        return $html;
        
	}


    // ----------------------------------------------------------------------
    // getConfig()
    // ----------------------------------------------------------------------
	function getConfig() {
	
        #global $ADMIN_CONF;
        #$language = $ADMIN_CONF->get("language");
        $config = array();

        $config['deDE']['ip_lock_sec'] = array(
            "type" => "text",
            "description" => "Dauer IP-Sperre (Sek. 86400 Sek. = 1 Tag)",
            "maxlength" => "6",
            "size" => "10",
            "regex" => "/^[0-9]{1,6}$/",
            "regex_error" => "Nur Ganzzahlen erlaubt (0 - 99999)."
        );
        
        $config['deDE']['ip_user_online_sec'] = array(
            "type" => "text",
            "description" => "Dauer User als online behandeln (Sek.)",
            "maxlength" => "6",
            "size" => "10",
            "regex" => "/^[0-9]{1,6}$/",
            "regex_error" => "Nur Ganzzahlen erlaubt (0 - 99999)."
        );

        $config['deDE']['list_prefix'] = array(
            "type" => "text",
            "description" => "Listen-Aufzählungszeichen",
            "maxlength" => "8",
            "size" => "10"
        );

        $config['deDE']['counter_userdefined'] = array(
            "type" => "textarea",
            "cols" => "50",
            "rows" => "10",
            "description" => "Benutzerdef. Counter-Darstellung",
        );

        #if(isset($config[$language])) {
        #  return $config[$language];
        #} else {
        return $config['deDE'];
        #}
	
	}


    // ----------------------------------------------------------------------
    // getInfo()
    // ----------------------------------------------------------------------
	function getInfo() {
	  #global $ADMIN_CONF;
	  #$language = $ADMIN_CONF->get("language");
		
	  $info['deDE'] = array(
	      // Plugin-Name
	      "<b>SimpleCounter</b> 2.0",
	      // CMS-Version
	      "2.0",
	      // Kurzbeschreibung
	      'Erzeugt einen einfachen Besucher-Counter. Es können dabei verschiedene Werte angezeigt werden.
	      Mittels IP-Sperre und Cookie wird versucht, im Rahmen des Möglichen eine Doppelzählung zu vermeiden.<br /> 
	      <br /> 
          Für weitere Infos - siehe "readme.htm" im Plugin-Ordner.<br /> 
          <br /> 
	      <span style="font-weight:bold;">Nutzung:</span><br />
	      In der im template.html oder Inhaltsseite an der gewünschten Stelle {SimpleCounter} oder
	      {SimpleCounter|hidden} angeben. Bei {SimpleCounter|hidden} wird die Anzeige unterdrückt,
	      aber dennoch gezählt. Achtung! Falls Sie den Counter nicht in die template.html geben:
	      Es wird nur auf Seiten gezählt, wo der Counter eingebunden ist!<br /> <br />
          
          <span style="font-weight:bold;">Benutzerdef. Counter-Darstellung (alternativ):</span><br />
	      Ist das benutzerdefinierte Feld unten leer, wird eine Standard-Counter-Ausgabe erstellt.<br />
	      Benötigt man nicht alle Werte, oder möchte man eine andere Reihenfolge, kann dies in dem Benutzerdef. Feld unten angegeben werden.
	      Folgende Detailwerte werden vom Plugin zur Verfügung gestellt und können unten im Feld alternativ verwendet werden:<br /> <br />

          {IconUndTitel|Mein Titel} Anzeige des Statistik-Icon und "Mein Titel" <br /> 
          {Titel|Mein Titel} Anzeige "Mein Titel" <br /> 
	      {UserOnline} Besucher die "aktuell" als online gelten<br />
          {Heute} Besucher Heute <br />
          {Woche} Besucher diese Woche <br />
          {Monat} Besucher diesen Monat <br />
          {Jahr} Besucher dieses Jahr <br />
          {Gesamt} Besucher gesamt <br />
          {Rekord} Maximale Tages-Besucher mit Datum<br /> 
          <br /> 
          Für weitere Infos - siehe "readme.htm" im Plugin-Ordner.<br />
          <br />
          <span style="font-weight:bold;">Aktuelle Counterwerte:</span><br />
          '. $this->getCounterValuesAsString(),
	      // Name des Autors
	      "hausl78 berichtigt von A. Fleischmann",
	      // Download-URL
	      "http://www.mozilo.info",
        array(
            '{SimpleCounter}' => 'SimpleCounter einfügen',
            '{SimpleCounter|hidden}' => 'SimpleCounter - Anzeige verstecken'
        )
	  );

	  #if(isset($info[$language])) {
	  #    return $info[$language];
	  #} else {
	  return $info['deDE'];
	  #}
	}


    // ----------------------------------------------------------------------
    // Aktuelle Zählerstände zur Anzeige im Plugin-Admin rausholen
    // ----------------------------------------------------------------------
    function getCounterValuesAsString() {
        // array aus file erstellen
        $arr_counter_data = array();
        $arr_counter_data = unserialize(file_get_contents($this->datafile_path_abs));
        if ($arr_counter_data === false) {
        	return 'Inhalt von "'.$this->datafile_path_rel.'" ist nicht interpretierbar!';
        }
        $counter_data = 
            'Heute: '  .$this->formatNumber($arr_counter_data['dayHits'])  .'<br />'.
            'Woche: '  .$this->formatNumber($arr_counter_data['weekHits']) .'<br />'.
            'Monat: '  .$this->formatNumber($arr_counter_data['monthHits']).'<br />'.
            'Jahr: '   .$this->formatNumber($arr_counter_data['yearHits']) .'<br />'.
            'Gesamt: ' .$this->formatNumber($arr_counter_data['allHits'])  .'<br />'.
            'Rekord: ' .$this->formatNumber($arr_counter_data['maxDayHits']). " (" . date("d.m.Y", $arr_counter_data['maxDayHits_timestamp']). ')<br />'
        ;
        return $counter_data;
    }


    // ----------------------------------------------------------------------
    // "Platzhalter" ersetzen und optionalen Text beibehalten
    // ----------------------------------------------------------------------
    function replacePlaceholder($haystack, $placeholder, $replace_to, $css_class) {
    	$regex = '#\{'. preg_quote($placeholder) .'(?:\|(.*))?\}#Us';
    	$replace = '<li class="'.$css_class.'">'.$replace_to."$1".'</li>';
    	return preg_replace($regex, $replace, $haystack);
    }


    // ----------------------------------------------------------------------
    // Hilfsfunktion: Zahlen formatieren
    // ----------------------------------------------------------------------
    function formatNumber($n) {
        $n = (int)$n;
        return number_format($n, 0, '', '.');
    }


    // ----------------------------------------------------------------------
    // Hilfsfunktion: Gibt Fehlermeldung aus
    // ----------------------------------------------------------------------
    function buildErrorMsg($msg) {
        return '<span class="deadlink">Fehler im Plugin SimpleCounter: '.$msg.'</span>';
    }
	
	function msec(){
		list($usec, $sec) = explode(' ',microtime());
		return intval(($usec+$sec)*1000.0);
	}
	
	function getIP(){
	
		$ip = $this->getRealIP();
		
		if (strstr($ip, ',')) {
			$ips = explode(',', $ip);
			$ip = trim($ips[0]);
		}
		
		return $ip;	
	}
	function getRealIP(){
	
		if (!empty($_SERVER["HTTP_CLIENT_IP"])) {
			return $_SERVER["HTTP_CLIENT_IP"];
		}
		
		if (!empty($_SERVER["HTTP_X_FORWARDED_FOR"])) {
			return $_SERVER["HTTP_X_FORWARDED_FOR"];
		}
		
		if (!empty($_SERVER["REMOTE_ADDR"])) {
			return $_SERVER["REMOTE_ADDR"];
		}
		return "noip";
	}
}
?>