<?php
/**
*
* $Rev: 56 $
* $Author: omgitsjustjava $
* $Date: 2016-06-14 21:36:53 +0200 (Di, 14 Jun 2016) $
*
*/
class audiovideo extends Plugin {
	
	// Versionsflag
	private $isVersionZweiNull;
	
	// unterstützte Dateiformate und ihre Mediatypes
	private $fileformats;
	
	function __construct() {
		parent::__construct();
		
		// welche CMS-Version haben wir?
		$this->isVersionZweiNull = (defined('CMSVERSION') && CMSVERSION == '2.0');

		// unterstützte Dateiformate und ihre Mimetypes
		$this->fileformats = array(
			'audio' => array(
				'mp3' => 'audio/mpeg',
				'ogg' => 'audio/ogg',
				'wav' => 'audio/wav'
			),
			'video' => array(
				'mp4' => 'video/mp4',
				'webm' => 'video/webm',
				'ogg' => 'video/ogg'
			)
		);
	}

    function getContent($value) {
		global $CatPage;
		global $specialchars;
		global $URL_BASE;
		global $PLUGIN_DIR_NAME;
		
		// Kein Parameter übergeben?
        if ($value == "") {
            return $this->buildErrorMsg("Kein Parameter übergeben.");
        }
		
		// Pfad zur Datei (URL)
		$filepath = '';
		
		// moziloCMS Version 2.0
		if ($this->isVersionZweiNull) {
			list($cat,$file) = $CatPage->split_CatPage_fromSyntax($value,true);
			if ($cat == "" && $file == "") {
				$values = explode("%3A",$value);
				if (count($values)==2) {
					$datei_text = $specialchars->rebuildSpecialChars($values[1],true,true);
					$cat_text   = $specialchars->rebuildSpecialChars($values[0],true,true);
				}else{
					$datei_text = "";
					$cat_text = "";
				}
				return $this->buildErrorMsg("Angeforderte Mediendatei ist nicht vorhanden!");	
			}
			if(!$CatPage->exists_File($cat,$file)) {
				return $this->buildErrorMsg("Angeforderte Mediendatei ist nicht vorhanden!");
			}
			$filepath = URL_BASE.'kategorien/'.str_replace("%","%25",$cat).'/dateien/'.$file;
		}
		// moziloCMS Version 1.12
		else {
			// Kategorie und Datei aus dem übergebenen Wert lesen
			list($cat, $file) = $CatPage->split_CatPage_fromSyntax($value, false, true);
			// Datei auf Existenz prüfen
			if(!$CatPage->exists_File($cat,$file)) {
				return $this->buildErrorMsg('Angeforderte Mediendatei ist nicht vorhanden!');
			}
			$cat = $CatPage->get_FileSystemName($cat, false);
			$filepath = $URL_BASE.'kategorien/'.str_replace("%","%25",$cat).'/dateien/'.$file;
		}
		
		// Pfad zur Datei (Filesystem)
		$filepathFilesystem = dirname(__FILE__).DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'kategorien'.DIRECTORY_SEPARATOR.$cat.DIRECTORY_SEPARATOR.'dateien'.DIRECTORY_SEPARATOR.$file;
		// Mimetype der Datei rausfinden
		$mimetype = $this->getMimeType($filepathFilesystem);
		if (!isset($mimetype)) {
			return $this->buildErrorMsg('Nicht unterstützte Dateiendung');
		}
		
		// abhängig vom Mimetype: Brauchen wir ein audio- oder ein video-Element?
		$mediatypeToUse = null;
		foreach ($this->fileformats as $currentMediaType => $currentMediaTypeFormats) {
			foreach($currentMediaTypeFormats as $currentExtension => $currentMimetype) {
				if ($mimetype == $currentMimetype) {
					$mediatypeToUse = $currentMediaType;
				}
			}
		}
		// wenn die Datei einen nicht unterstützten Mimetype hat
		if (!isset($mediatypeToUse)) {
			return $this->buildErrorMsg('Nicht unterstützter Dateityp '.$mimetype);
		}
		
		// audio- bzw. video-Element ausgeben
		return '<'.$mediatypeToUse.' class="audiovideo" type="'.$mimetype.'" src="'.$filepath.'" controls>Kann Mediendatei nicht abspielen - <a href="'.$filepath.'">stattdessen herunterladen</a></'.$mediatypeToUse.'>';
    } // function getContent
	
    
    function getConfig() {
		global $CatPage;
		global $specialchars;
		global $URL_BASE;
		global $PLUGIN_DIR_NAME;
		global $BASE_DIR;
		
		// (Bisher) nix zu konfiguerieren :)
		$config = array();
		return $config;
    } // function getConfig    

    function getInfo() {
        global $ADMIN_CONF;
        $language = $ADMIN_CONF->get("language");
        # nur eine Sprache ---------------------------------
		$prefix = $this->isVersionZweiNull ? '@=' : '';
		$suffix = $this->isVersionZweiNull ? '=@' : '';
		$description = 'Ein einfacher Player für Audio- und Video-Dateien.<br /><br /><span style="font-weight:bold;">Verwendung:</span><br />{audiovideo|'.$prefix.'dateiname.mp3'.$suffix.'}<br />bzw.<br />{audiovideo|'.$prefix.'Kategorie:dateiname.mp3'.$suffix.'}<br /><br /><span style="font-weight:bold;">Unterstützte Dateitypen:</span>';
		foreach (array('audio', 'video') as $type) {
			$description .= '<br />' . ucfirst($type) . ':';
			foreach ($this->fileformats[$type] as $format => $mediatype) {
				$description .= '<br />- ' . $format . ' (' . $mediatype . ')';
			}
		}
		$rev = preg_replace("/[^0-9]/","",'$Rev: 56 $');
        $info = array(
            // Plugin-Name + Version
            '<b>Audio- und Videoplayer</b> Revision '.$rev,
            // moziloCMS-Version
            '1.12 / 2.0',
            // Kurzbeschreibung nur <span> und <br /> sind erlaubt
			$description,
            // Name des Autors
            'Arvid Zimmermann',
            // Download-URL
            'http://www.arvidzimmermann.de',
            // Platzhalter für die Selectbox in der Editieransicht 
            // - ist das Array leer, erscheint das Plugin nicht in der Selectbox
            array(
                '{audiovideo|}' => 'Player für Audio- und Video-Dateien'
            )
        );
        // Rückgabe der Infos.
        // Auch hier könnte man die Inhalte natürlich von der aktuell im Admin eingestellten 
        // Sprache abhängig machen - siehe getConfig().
        return $info;
        
    } // function getInfo
	
	// Zusatzfunktion: Gibt Fehlermeldung aus
	function buildErrorMsg($msg) {
		return '<span class="deadlink">Fehler im Plugin '.get_class($this).': '.$msg.'</span>';
	}
	
	/**
	* Gibt den Mimetype der übergebenen Datei zurück.
	* 
	* @param string $filepath der absolute Pfad zur Datei
	* @return string der Mimetype - oder <code>null</code>, wenn die Datei einen unbekannten Typ hat
	*/
	function getMimeType($filepath) {
		// Der saubere Weg ab PHP 5.1.0: Per FileInfo
		if (function_exists('finfo_open')) {
			$finfo = finfo_open(FILEINFO_MIME);
			$mimetype = finfo_file($finfo, $filepath);
			$mimetype = substr($mimetype, 0, strpos($mimetype, ';'));
			finfo_close($finfo);
			return $mimetype;
		}
		// Vor PHP 5.1.: Per mime_content_type()
		elseif(function_exists('mime_content_type')) {
			return mime_content_type($filepath);
		}
		// wenn auch die nicht auf dem Server vorhanden ist, machen wirs von Hand:
		else {
			// Extension der Datei holen
			$ext = strtolower(array_pop(explode('.', $filepath)));
			// Im fileformats-Array suchen ('ogg' matcht dann halt zuerst als Audio...)
			foreach ($this->fileformats as $currentMediaType => $currentMediaTypeFormats) {
				if (array_key_exists($ext, $currentMediaTypeFormats)) {
					return $currentMediaTypeFormats[$ext];
				}
			}
		}
		// nicht gefunden?
		return null;
	}

} // class audiovideo

?>